{% extends 'dashboard/layout.html' %}
{% load i18n %}

{% block body_class %}cancel-docdata-order{% endblock %}

{% block title %}
    {% blocktrans with title=object.title %}Cancel Docdata Order '{{ title }}'?{% endblocktrans %}
    | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <ul class="breadcrumb">
        <li>
            <a href="{% url 'dashboard:index' %}">{% trans "Dashboard" %}</a>
            <span class="divider">/</span>
        </li>
        <li>
            <a href="{% url 'docdata-order-list' %}">{% trans "Docdata Orders" %}</a>
            <span class="divider">/</span>
        </li>
        <li class="active">{% blocktrans with title=object.title %}Cancel Docdata Order '{{ title }}'?{% endblocktrans %}</li>
    </ul>
{% endblock %}

{% block headertext %}
    {% blocktrans with title=object.title %}Cancel Docdata Order '{{ title }}'?{% endblocktrans %}
{% endblock %}

{% block dashboard_content %}
    <div class="table-header">
        <h2>{% trans "Cancel Docdata Order" %}</h2>
    </div>
    <form action="." method="post" class="well">
        {% csrf_token %}
        {{ form }}

        <p>{% blocktrans with title=object.merchant_order_id %}Cancel Docdata Order <strong>{{ title }}</strong> - are you sure?{% endblocktrans %}</p>
        <div style="border: 1px solid red; color: red; padding: 5px; margin: 0 0 1em 0; max-width: 500px;">
          {% trans 'Warning: this will cancel the entire order!' %}
        </div>

        <div class="form-actions">
            <button type="submit" class="btn btn-danger">{% trans "Cancel" %}</button>
            {% trans "or" %} <a href="{% url 'docdata-order-detail' pk=object.pk %}">{% trans "abort" %}</a>.
        </div>
    </form>
{% endblock %}
